require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
  class Game
    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end

    def load_players(file)
      CSV.foreach(file) do |line|
        add_player(Player.from_csv(line))
      end
    end

    def save_high_scores(file = "high_scores.txt")
      File.open(file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each do |player|
          file.puts high_score_entry(player)
        end
      end
    end

    def add_player(player)
      @players << player
    end

    def play(rounds)
      1.upto(rounds) do |round|
        if block_given?
          break if yield
        end
        puts "\nRound #{round}:"
        @players.each do |player|
          GameTurn.take_turn(player)
        end
      end

      treasures = TreasureTrove::TREASURES

      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end
    end

    def print_name_and_health(player)
      puts "#{player.name} (#{player.health})"
    end

    def total_points
      @players.reduce(0) {|sum, player| sum + player.points}
    end

    def high_score_entry(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end

    def print_stats
      strong, wimpy = @players.partition {|player| player.strong?}
      puts "\n#{@title} Statistics:"

      puts "\n#{strong.size} strong players:"
      strong.each {|player| print_name_and_health(player)}

      puts "\n#{wimpy.size} wimpy players:"
      wimpy.each {|player| print_name_and_health(player)}

      puts "\n#{@title} High Scores:"
      @players.sort.each do |player|
        puts high_score_entry(player)
      end

      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure {|treasure| puts "#{treasure.points} total #{treasure.name} points"}
        puts "#{player.points} grand total points"
      end

      puts "\n#{total_points} total points from treasures found"
    end
  end
end